@@ -127,6 +127,7 @@ struct Dav1dContext {
     int operating_point;
     unsigned operating_point_idc;
     int all_layers;
+    unsigned frame_size_limit;
     int drain;
 
     Dav1dLogger logger;
@@ -195,12 +196,12 @@ struct Dav1dFrameContext {
             int16_t eob[3 /* plane */];
             uint8_t txtp[3 /* plane */];
         } *cbi;
-        int8_t *txtp;
         // indexed using (t->by >> 1) * (f->b4_stride >> 1) + (t->bx >> 1)
         uint16_t (*pal)[3 /* plane */][8 /* idx */];
         // iterated over inside tile state
         uint8_t *pal_idx;
         coef *cf;
+        int pal_sz, pal_idx_sz, cf_sz;
         // start offsets per tile
         int *tile_start_off;
     } frame_thread;
@@ -216,10 +217,8 @@ struct Dav1dFrameContext {
         int last_sharpness;
         uint8_t lvl[8 /* seg_id */][4 /* dir */][8 /* ref */][2 /* is_gmv */];
         uint8_t *tx_lpf_right_edge[2];
-        pixel *cdef_line;
-        pixel *cdef_line_ptr[2 /* pre, post */][3 /* plane */][2 /* y */];
-        pixel *lr_lpf_line;
-        pixel *lr_lpf_line_ptr[3 /* plane */];
+        pixel *cdef_line[2 /* pre, post */][3 /* plane */][2 /* y */];
+        pixel *lr_lpf_line[3 /* plane */];
 
         // in-loop filter per-frame state keeping
         int tile_row; // for carry-over at tile row edges
@@ -274,24 +273,54 @@ struct Dav1dTileContext {
     Dav1dTileState *ts;
     int bx, by;
     BlockContext l, *a;
-    coef *cf;
-    pixel *emu_edge; // stride=192 for non-SVC, or 320 for SVC
+    ALIGN(union, 32) {
+        int16_t cf_8bpc [32 * 32];
+        int32_t cf_16bpc[32 * 32];
+    };
     // FIXME types can be changed to pixel (and dynamically allocated)
     // which would make copy/assign operations slightly faster?
     uint16_t al_pal[2 /* a/l */][32 /* bx/y4 */][3 /* plane */][8 /* palette_idx */];
-    ALIGN(uint16_t pal[3 /* plane */][8 /* palette_idx */], 16);
     uint8_t pal_sz_uv[2 /* a/l */][32 /* bx4/by4 */];
     uint8_t txtp_map[32 * 32]; // inter-only
-    Dav1dWarpedMotionParams warpmv;
-    union {
-        void *mem;
-        uint8_t *pal_idx;
-        int16_t *ac;
-        pixel *interintra, *lap;
-        int16_t *compinter;
+    ALIGN(union, 32) {
+        struct {
+            union {
+                uint8_t  lap_8bpc [128 * 32];
+                uint16_t lap_16bpc[128 * 32];
+                struct {
+                    int16_t compinter[2][128 * 128];
+                    uint8_t seg_mask[128 * 128];
+                };
+            };
+            union {
+                // stride=192 for non-SVC, or 320 for SVC
+                uint8_t  emu_edge_8bpc [320 * (256 + 7)];
+                uint16_t emu_edge_16bpc[320 * (256 + 7)];
+            };
+        };
+        struct {
+            uint8_t interintra_8bpc[64 * 64];
+            uint8_t edge_8bpc[257];
+        };
+        struct {
+            uint16_t interintra_16bpc[64 * 64];
+            uint16_t edge_16bpc[257];
+        };
+        struct {
+            uint8_t pal_idx[2 * 64 * 64];
+            union {
+                struct {
+                    uint8_t pal_order[64][8];
+                    uint8_t pal_ctx[64];
+                };
+                uint8_t levels[36 * 36];
+            };
+            uint16_t pal[3 /* plane */][8 /* palette_idx */];
+        };
+        int16_t ac[32 * 32];
     } scratch;
-    ALIGN(uint8_t scratch_seg_mask[128 * 128], 32);
 
+    Dav1dWarpedMotionParams warpmv;
     Av1Filter *lf_mask;
     int8_t *cur_sb_cdef_idx_ptr;
     // for chroma sub8x8, we need to know the filter for all 4 subblocks in
