@@ -635,10 +635,8 @@ static void mask_c(pixel *dst, const ptrdiff_t dst_stride,
 }
 
 #define blend_px(a, b, m) (((a * (64 - m) + b * m) + 32) >> 6)
-static NOINLINE void
-blend_internal_c(pixel *dst, const ptrdiff_t dst_stride, const pixel *tmp,
-                 const int w, int h, const uint8_t *mask,
-                 const ptrdiff_t mask_stride)
+static void blend_c(pixel *dst, const ptrdiff_t dst_stride, const pixel *tmp,
+                    const int w, int h, const uint8_t *mask)
 {
     do {
         for (int x = 0; x < w; x++) {
@@ -646,26 +644,28 @@ blend_internal_c(pixel *dst, const ptrdiff_t dst_stride, const pixel *tmp,
         }
         dst += PXSTRIDE(dst_stride);
         tmp += w;
-        mask += mask_stride;
+        mask += w;
     } while (--h);
 }
 
-static void blend_c(pixel *dst, const ptrdiff_t dst_stride, const pixel *tmp,
-                    const int w, const int h, const uint8_t *mask)
-{
-    blend_internal_c(dst, dst_stride, tmp, w, h, mask, w);
-}
-
 static void blend_v_c(pixel *dst, const ptrdiff_t dst_stride, const pixel *tmp,
-                      const int w, const int h)
+                      const int w, int h)
 {
-    blend_internal_c(dst, dst_stride, tmp, w, h, &dav1d_obmc_masks[w], 0);
+    const uint8_t *const mask = &dav1d_obmc_masks[w];
+    do {
+        for (int x = 0; x < (w * 3) >> 2; x++) {
+            dst[x] = blend_px(dst[x], tmp[x], mask[x]);
+        }
+        dst += PXSTRIDE(dst_stride);
+        tmp += w;
+    } while (--h);
 }
 
 static void blend_h_c(pixel *dst, const ptrdiff_t dst_stride, const pixel *tmp,
                       const int w, int h)
 {
     const uint8_t *mask = &dav1d_obmc_masks[h];
+    h = (h * 3) >> 2;
     do {
         const int m = *mask++;
         for (int x = 0; x < w; x++) {
