@@ -27,7 +27,6 @@
 
 #include "config.h"
 
-#include <assert.h>
 #include <limits.h>
 
 #include "common/intops.h"
@@ -68,7 +67,7 @@ static inline void ctx_norm(MsacContext *s, ec_win dif, unsigned rng) {
         ctx_refill(s);
 }
 
-unsigned dav1d_msac_decode_bool_equi(MsacContext *const s) {
+unsigned dav1d_msac_decode_bool_equi_c(MsacContext *const s) {
     ec_win vw, dif = s->dif;
     unsigned ret, v, r = s->rng;
     assert((dif >> (EC_WIN_SIZE - 16)) < r);
@@ -86,7 +85,7 @@ unsigned dav1d_msac_decode_bool_equi(MsacContext *const s) {
 /* Decode a single binary value.
  * f: The probability that the bit is one
  * Return: The value decoded (0 or 1). */
-unsigned dav1d_msac_decode_bool(MsacContext *const s, const unsigned f) {
+unsigned dav1d_msac_decode_bool_c(MsacContext *const s, const unsigned f) {
     ec_win vw, dif = s->dif;
     unsigned ret, v, r = s->rng;
     assert((dif >> (EC_WIN_SIZE - 16)) < r);
@@ -99,13 +98,6 @@ unsigned dav1d_msac_decode_bool(MsacContext *const s, const unsigned f) {
     return !ret;
 }
 
-unsigned dav1d_msac_decode_bools(MsacContext *const s, unsigned n) {
-    unsigned v = 0;
-    while (n--)
-        v = (v << 1) | dav1d_msac_decode_bool_equi(s);
-    return v;
-}
-
 int dav1d_msac_decode_subexp(MsacContext *const s, const int ref,
                              const int n, const unsigned k)
 {
@@ -122,15 +114,6 @@ int dav1d_msac_decode_subexp(MsacContext *const s, const int ref,
                           n - 1 - inv_recenter(n - 1 - ref, v);
 }
 
-int dav1d_msac_decode_uniform(MsacContext *const s, const unsigned n) {
-    assert(n > 0);
-    const int l = ulog2(n) + 1;
-    assert(l > 1);
-    const unsigned m = (1 << l) - n;
-    const unsigned v = dav1d_msac_decode_bools(s, l - 1);
-    return v < m ? v : (v << 1) - m + dav1d_msac_decode_bool_equi(s);
-}
-
 /* Decodes a symbol given an inverse cumulative distribution function (CDF)
  * table in Q15. */
 static unsigned decode_symbol(MsacContext *const s, const uint16_t *const cdf,
@@ -172,8 +155,8 @@ unsigned dav1d_msac_decode_symbol_adapt_c(MsacContext *const s,
     return val;
 }
 
-unsigned dav1d_msac_decode_bool_adapt(MsacContext *const s,
-                                      uint16_t *const cdf)
+unsigned dav1d_msac_decode_bool_adapt_c(MsacContext *const s,
+                                        uint16_t *const cdf)
 {
     const unsigned bit = dav1d_msac_decode_bool(s, *cdf);
 
@@ -181,11 +164,10 @@ unsigned dav1d_msac_decode_bool_adapt(MsacContext *const s,
         // update_cdf() specialized for boolean CDFs
         const unsigned count = cdf[1];
         const int rate = (count >> 4) | 4;
-        if (bit) {
+        if (bit)
             cdf[0] += (32768 - cdf[0]) >> rate;
-        } else {
+        else
             cdf[0] -= cdf[0] >> rate;
-        }
         cdf[1] = count + (count < 32);
     }
 
