@@ -1297,10 +1297,20 @@ int dav1d_parse_obus(Dav1dContext *const c, Dav1dData *const in, int global) {
                 c->frame_hdr = NULL;
                 return DAV1D_ERR(EINVAL);
             }
-
-            break;
         }
-        // OBU_FRAMEs shouldn't be signalled with show_existing_frame
+
+        if (c->frame_size_limit && (int64_t)c->frame_hdr->width[1] *
+            c->frame_hdr->height > c->frame_size_limit)
+        {
+            dav1d_log(c, "Frame size %dx%d exceeds limit %u\n", c->frame_hdr->width[1],
+                      c->frame_hdr->height, c->frame_size_limit);
+            c->frame_hdr = NULL;
+            return DAV1D_ERR(ERANGE);
+        }
+
+        if (type != OBU_FRAME)
+            break;
+        // OBU_FRAMEs shouldn't be signaled with show_existing_frame
         if (c->frame_hdr->show_existing_frame) {
             c->frame_hdr = NULL;
             goto error;
@@ -1418,6 +1428,7 @@ int dav1d_parse_obus(Dav1dContext *const c, Dav1dData *const in, int global) {
         default:
             // print a warning but don't fail for unknown types
             dav1d_log(c, "Unknown Metadata OBU type %d\n", meta_type);
+            break;
         }
 
         break;
@@ -1427,8 +1438,9 @@ int dav1d_parse_obus(Dav1dContext *const c, Dav1dData *const in, int global) {
         // ignore OBUs we don't care about
         break;
     default:
+        // print a warning but don't fail for unknown types
         dav1d_log(c, "Unknown OBU type %d of size %u\n", type, len);
-        return DAV1D_ERR(EINVAL);
+        break;
     }
 
     if (c->seq_hdr && c->frame_hdr) {
