@@ -100,6 +100,8 @@ static const struct {
     { "AVX-512", "avx512", DAV1D_X86_CPU_FLAG_AVX512 },
 #elif ARCH_AARCH64 || ARCH_ARM
     { "NEON",    "neon",   DAV1D_ARM_CPU_FLAG_NEON },
+#elif ARCH_PPC64LE
+    { "VSX",     "vsx",    DAV1D_PPC_CPU_FLAG_VSX },
 #endif
     { 0 }
 };
@@ -136,6 +138,8 @@ static struct {
     const char *cpu_flag_name;
     const char *test_name;
     unsigned int seed;
+    int bench_c;
+    int verbose;
 } state;
 
 /* float compare support code */
@@ -342,7 +346,7 @@ static void print_benchs(const CheckasmFunc *const f) {
         print_benchs(f->child[0]);
 
         /* Only print functions with at least one assembly version */
-        if (f->versions.cpu || f->versions.next) {
+        if (state.bench_c || f->versions.cpu || f->versions.next) {
             const CheckasmFuncVersion *v = &f->versions;
             do {
                 if (v->iterations) {
@@ -502,7 +506,20 @@ int main(int argc, char *argv[]) {
     int ret = 0;
 
     while (argc > 1) {
-        if (!strncmp(argv[1], "--bench", 7)) {
+        if (!strncmp(argv[1], "--help", 6)) {
+            fprintf(stdout,
+                    "checkasm [options] <random seed>\n"
+                    "    <random seed>       Numeric value to seed the rng\n"
+                    "Options:\n"
+                    "    --test=<test_name>  Test only <test_name>\n"
+                    "    --bench=<pattern>   Test and benchmark the functions matching <pattern>\n"
+                    "    --list              List the available tests\n"
+                    "    --bench-c           Benchmark the C-only functions\n"
+                    "    --verbose -v        Print failures verbosely\n");
+            return 0;
+        } else if (!strncmp(argv[1], "--bench-c", 9)) {
+            state.bench_c = 1;
+        } else if (!strncmp(argv[1], "--bench", 7)) {
 #ifndef readtime
             fprintf(stderr,
                     "checkasm: --bench is not supported on your system\n");
@@ -521,6 +538,8 @@ int main(int argc, char *argv[]) {
                 fprintf(stderr, "%s%s", i ? ", ": "", tests[i].name);
             fprintf(stderr, "]\n");
             return 0;
+        } else if (!strcmp(argv[1], "--verbose") || !strcmp(argv[1], "-v")) {
+            state.verbose = 1;
         } else {
             state.seed = (unsigned int) strtoul(argv[1], NULL, 10);
         }
@@ -597,7 +616,7 @@ void *checkasm_check_func(void *const func, const char *const name, ...) {
     state.current_func_ver = v;
     xor128_srand(state.seed);
 
-    if (state.cpu_flag)
+    if (state.cpu_flag || state.bench_c)
         state.num_checked++;
 
     return ref;
@@ -610,8 +629,9 @@ int checkasm_bench_func(void) {
                     state.bench_pattern_len);
 }
 
-/* Indicate that the current test has failed */
-void checkasm_fail_func(const char *const msg, ...) {
+/* Indicate that the current test has failed, return whether verbose printing
+ * is requested. */
+int checkasm_fail_func(const char *const msg, ...) {
     if (state.current_func_ver->cpu && state.current_func_ver->ok) {
         va_list arg;
 
@@ -626,6 +646,7 @@ void checkasm_fail_func(const char *const msg, ...) {
         state.current_func_ver->ok = 0;
         state.num_failed++;
     }
+    return state.verbose;
 }
 
 /* Update benchmark results of the current function */
@@ -687,3 +708,41 @@ void checkasm_set_signal_handler_state(const int enabled) {
     signal(SIGSEGV, handler);
 #endif
 }
+
+#define DEF_CHECKASM_CHECK_FUNC(type, fmt) \
+int checkasm_check_##type(const char *const file, const int line, \
+                          const type *buf1, ptrdiff_t stride1, \
+                          const type *buf2, ptrdiff_t stride2, \
+                          const int w, int h, const char *const name) \
+{ \
+    stride1 /= sizeof(*buf1); \
+    stride2 /= sizeof(*buf2); \
+    int y = 0; \
+    for (y = 0; y < h; y++) \
+        if (memcmp(&buf1[y*stride1], &buf2[y*stride2], w*sizeof(*buf1))) \
+            break; \
+    if (y == h) \
+        return 0; \
+    if (!checkasm_fail_func("%s:%d", file, line)) \
+        return 1; \
+    fprintf(stderr, "%s:\n", name); \
+    while (h--) { \
+        for (int x = 0; x < w; x++) \
+            fprintf(stderr, " " fmt, buf1[x]); \
+        fprintf(stderr, "    "); \
+        for (int x = 0; x < w; x++) \
+            fprintf(stderr, " " fmt, buf2[x]); \
+        fprintf(stderr, "    "); \
+        for (int x = 0; x < w; x++) \
+            fprintf(stderr, "%c", buf1[x] != buf2[x] ? 'x' : '.'); \
+        buf1 += stride1; \
+        buf2 += stride2; \
+        fprintf(stderr, "\n"); \
+    } \
+    return 1; \
+}
+
+DEF_CHECKASM_CHECK_FUNC(uint8_t,  "%02x")
+DEF_CHECKASM_CHECK_FUNC(uint16_t, "%04x")
+DEF_CHECKASM_CHECK_FUNC(int16_t,  "%6d")
+DEF_CHECKASM_CHECK_FUNC(int32_t,  "%9d")
